/*******************************************************************************
 * Copyright 2022 ModalAI Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. The Software is used solely in conjunction with devices provided by
 *    ModalAI Inc.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/


#ifndef MODALCV_DCM_H
#define MODALCV_DCM_H


#ifdef __cplusplus
extern "C" {
#endif


////////////////////////////////////////////////////////////////////////////////
// Feature Descriptor Calculation and Match on CVP
////////////////////////////////////////////////////////////////////////////////


#define MCV_DCM_MAX_DESCRIPTORS 1024

typedef struct mcv_dcm_pos_t{
	uint16_t x;
	uint16_t y;
} __attribute__((packed)) mcv_dcm_pos_t;

typedef struct mcv_dcm_match_t{
	uint16_t index; // ID within the descriptor list
	uint16_t score; // max score is 63
} __attribute__((packed)) mcv_dcm_match_t;

#define MCV_MAX_POS_BUF_SIZE (sizeof(mcv_dcm_pos_t)*MCV_DCM_MAX_DESCRIPTORS)
#define MCV_MAX_MATCH_BUF_SIZE (sizeof(mcv_dcm_match_t)*MCV_DCM_MAX_DESCRIPTORS)


// allocate required_buf_size for points before calling fpx_process
int mcv_dcm_init(int width, int height);
int mcv_dcm_deinit(void);

int mcv_dcm_calc(uint8_t* img, mcv_dcm_pos_t* positions, int n_pos);
int mcv_dcm_match(uint8_t* img, mcv_dcm_pos_t* positions, int n_pos,\
						mcv_dcm_match_t* matches, int* n_matches);

int mcv_dcm_reset_descriptors();


#ifdef __cplusplus
}
#endif

#endif // end #define MODALCV_FPX_H