/*******************************************************************************
 * Copyright 2022 ModalAI Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. The Software is used solely in conjunction with devices provided by
 *    ModalAI Inc.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/


#ifndef MODALCV_DFS_H
#define MODALCV_DFS_H


#ifdef __cplusplus
extern "C" {
#endif


#include <stdint.h>


////////////////////////////////////////////////////////////////////////////////
// Depth From Stereo DFS on CVP
////////////////////////////////////////////////////////////////////////////////


// Max Disparity must be multiple of 4 and is dependent on image width
//    If the width <= 640, max disparity can be 64, 60, 56, ..., 4.
//    If the width <= 1280, max disparity can be 32, 28, ..., 4.
//    If the width <= 1920, max disparity can be 16, 12, ..., 4.

typedef struct mcv_dfs_config_t{
	int width;
	int height;
	int max_disparity;
	int disparity_offset; // Valid range is [0, 254-nMaxDisparity]
	int en_median_filter;
	int en_occlusion_filling;
	int occlusion_min_threshold;	///< [0-100] nOcclusionMinThreshold <= nOcclusionThreshold
	int occlusion_cost_threshold;	///< [0-100] nOcclusionMinThreshold <= nOcclusionThreshold
} mcv_dfs_config_t;


int mcv_dfs_init(mcv_dfs_config_t config);
int mcv_dfs_deinit(void);

// disparity and occlusion masks are in a 4x4 raster, use other functions to
// convert to other desired output formats
int mcv_dfs_process(uint8_t* left, uint8_t* right, uint8_t* disparity, uint8_t* occlusion_mask);

// linearize the 4x4 raster output from dfs to a normal image
// this is slow, avoid if possible.
int mcv_dfs_linearize_disparity(uint8_t* disp_in, int w, int h, uint8_t* disp_out);
int mcv_dfs_linearize_occlusion(uint8_t* occ_in, int w, int h, uint8_t* occ_out);




#ifdef __cplusplus
}
#endif

#endif // end #define MODALCV_DFS_H