/*******************************************************************************
 * Copyright 2022 ModalAI Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * 4. The Software is used solely in conjunction with devices provided by
 *    ModalAI Inc.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/


#ifndef MODALCV_FPX_H
#define MODALCV_FPX_H


#ifdef __cplusplus
extern "C" {
#endif


////////////////////////////////////////////////////////////////////////////////
// Feature Point Extraction on CVP
////////////////////////////////////////////////////////////////////////////////

typedef enum mcv_fpx_mode_t{
	MCV_FPX_PEAK_8x8 = 0,
	MCV_FPX_ZONE
}mcv_fpx_mode_t;

typedef enum mcv_fpx_nms_t{
	MCV_FPX_3_TAP_NMS = 0,
	MCV_FPX_5_TAP_NMS
}mcv_fpx_nms_t;




typedef struct mcv_fpx_config_t{
	int width;
	int height;
	mcv_fpx_mode_t mode;
	mcv_fpx_nms_t nms_mode;
	uint32_t score_threshold;
	uint8_t robustness; // 0-127, default 10
}mcv_fpx_config_t;


typedef struct mcv_fpx_feature_t{
	uint16_t x;
	uint16_t y;
	uint16_t score;
	uint16_t reserved;
}__attribute__((packed)) mcv_fpx_feature_t;


// allocate required_buf_size for points before calling fpx_process
int mcv_fpx_init(mcv_fpx_config_t config, int* required_buf_size);
int mcv_fpx_deinit(void);

int mcv_fpx_process(uint8_t* img, mcv_fpx_feature_t* features, uint32_t* max_score, int* n_points);



#ifdef __cplusplus
}
#endif

#endif // end #define MODALCV_FPX_H